@@ -102,8 +102,6 @@ CHARACTER_IGN
 CHARACTER_IGN_REV
 CHARACTER_REV
 CHECK
-CHECK_IGN
-CHECK_IGN_REV
 CHECK_REV
 DEFAULT_BOUNDARY
 END
@@ -578,12 +576,7 @@ def _flatten_code(code):
     "Flattens the code from a list of tuples."
     flat_code = []
     for c in code:
-        if c[0] < 0:
-            # Negative opcodes are end-markers.
-            flat_code.append(_OP.END)
-        else:
-            flat_code.append(c[0])
-        flat_code.extend(c[1 : ])
+        flat_code.extend(c)
     return flat_code
 
 def _parse_pattern(source, info):
@@ -1324,19 +1317,9 @@ def _compile_repl_group(source, pattern):
 # node are defined below.
 
 def prefix_checks(checks, parsed):
-    if checks:
-        included = []
-        prefix = []
-        for ign, chars in sorted(checks, key=lambda m: len(m[1]), reverse=True):
-            s = u"".join(unichr(ch) for ch in chars)
-            if not any(s in inc for inc in included):
-                if ign:
-                    prefix.append(_CheckIgn(chars))
-                else:
-                    prefix.append(_Check(chars))
-                included.append(s)
-        parsed = _Sequence(prefix + [parsed])
-    return parsed
+    if isinstance(checks, _Sequence) and not checks.sequence:
+        return parsed
+    return _Check(checks, parsed)
 
 _INDENT = "  "
 _ZEROWIDTH_OP = 0x2
@@ -1382,7 +1365,7 @@ class _RegexBase(object):
     def __eq__(self, other):
         return type(self) is type(other) and self._key == other._key
     def get_checks(self, reverse):
-        return set()
+        return _Sequence()
 
 # Base for zero-width nodes.
 class _ZeroWidthBase(_RegexBase):
@@ -1444,7 +1427,7 @@ class _Atomic(_StructureBase):
     def contains_group(self):
         return self.subpattern.contains_group()
     def compile(self, reverse=False):
-        return [(_OP.ATOMIC, )] + self.subpattern.compile(reverse) + [(-_OP.ATOMIC, )]
+        return [(_OP.ATOMIC, )] + self.subpattern.compile(reverse) + [(_OP.END, )]
     def dump(self, indent=0, reverse=False):
         print "%s%s" % (_INDENT * indent, "ATOMIC")
         self.subpattern.dump(indent + 1, reverse)
@@ -1568,7 +1551,7 @@ class _Branch(_StructureBase):
         for branch in self.branches:
             code.extend(branch.compile(reverse))
             code.append((_OP.NEXT, ))
-        code[-1] = (-_OP.BRANCH, )
+        code[-1] = (_OP.END, )
         return code
     def remove_captures(self):
         self.branches = [branch.remove_captures() for branch in self.branches]
@@ -1585,11 +1568,17 @@ class _Branch(_StructureBase):
             fs |= branch.firstset()
         return fs or set([None])
     def get_checks(self, reverse):
-        checks = [b.get_checks(reverse) for b in self.branches]
-        common = set()
-        for c in checks:
-            common &= c
-        return common
+        branches = []
+        for b in self.branches:
+            b = b.get_checks(reverse)
+            if isinstance(b, _Sequence) and not b.sequence:
+                return b
+            branches.append(b)
+        if not branches:
+            return _Sequence()
+        if len(branches) == 1:
+            return branches[0]
+        return _Branch(branches)
     @staticmethod
     def _flush_char_prefix(info, char_type, prefixed, order, new_branches):
         for value, branches in sorted(prefixed.items(), key=lambda pair: order[pair[0]]):
@@ -1638,8 +1627,8 @@ class _Character(_RegexBase):
         return set([self])
     def get_checks(self, reverse):
         if not self.positive:
-            return set()
-        return set([(False, (self.value, ))])
+            return _Sequence()
+        return self
     def has_simple_start(self):
         return True
     def is_case_sensitive(self, info):
@@ -1659,32 +1648,27 @@ class _CharacterIgn(_Character):
         if self.is_case_sensitive(info):
             return self
         return _Character(self.value, positive=self.positive, zerowidth=self.zerowidth)
-    def get_checks(self, reverse):
-        if not self.positive:
-            return set()
-        return set([(True, (self.value, ))])
 
 class _Check(_RegexBase):
     _opcode = {False: _OP.CHECK, True: _OP.CHECK_REV}
     _op_name = {False: "CHECK", True: "CHECK_REV"}
-    def __init__(self, characters):
-        self.characters = tuple(characters)
-        self._key = self.__class__, self.characters
+    def __init__(self, check, body):
+        self.check, self.body = check, body
+        self._key = self.__class__
     def compile(self, reverse=False):
-        return [(self._opcode[reverse], len(self.characters)) + tuple(self.characters)]
+        code = self.check.compile(reverse)
+        if code:
+            code = [(self._opcode[reverse], )] + code + [(_OP.END, )]
+        return code + self.body.compile(reverse)
     def dump(self, indent=0, reverse=False):
-        print "%s%s %s" % (_INDENT * indent, self._op_name[reverse], " ".join(str(ch) for ch in self.characters))
-    def has_simple_start(self):
-        return True
+        print "%s%s" % (_INDENT * indent, self._op_name[reverse])
+        self.check.dump(indent + 1, reverse)
+        self.body.dump(indent, reverse)
     def get_first_char(self):
         raise error("internal error")
     def drop_first_char(self):
         raise error("internal error")
 
-class _CheckIgn(_Check):
-    _opcode = {False: _OP.CHECK_IGN, True: _OP.CHECK_IGN_REV}
-    _op_name = {False: "CHECK_IGN", True: "CHECK_IGN_REV"}
-
 class _Conditional(_StructureBase):
     def __init__(self, info, group, yes_item, no_item):
         self.info, self.group, self.yes_item, self.no_item = info, group, yes_item, no_item
@@ -1722,7 +1706,7 @@ class _Conditional(_StructureBase):
         if add_code:
             code.append((_OP.NEXT, ))
             code.extend(add_code)
-        code.append((-_OP.GROUP_EXISTS, ))
+        code.append((_OP.END, ))
         return code
     def remove_captures(self):
         self.yes_item = self.yes_item.remove_captures()
@@ -1737,7 +1721,7 @@ class _Conditional(_StructureBase):
     def firstset(self):
         return self.yes_item.firstset() | self.no_item.firstset()
     def get_checks(self, reverse):
-        return self.yes_item.get_checks(reverse) & self.no_item.get_checks(reverse)
+        return _Branch([self.yes_item.get_checks(reverse), self.no_item.get_checks(reverse)])
     def __eq__(self, other):
         return type(self) is type(other) and (self.group, self.yes_item, self.no_item) == (other.group, other.yes_item, other.no_item)
 
@@ -1795,7 +1779,7 @@ class _GreedyRepeat(_StructureBase):
             repeat.append(_UNLIMITED)
         else:
             repeat.append(self.max_count)
-        return [tuple(repeat)] + self.subpattern.compile(reverse) + [(-self._opcode, )]
+        return [tuple(repeat)] + self.subpattern.compile(reverse) + [(_OP.END, )]
     def remove_captures(self):
         self.subpattern = self.subpattern.remove_captures()
         return self
@@ -1812,7 +1796,7 @@ class _GreedyRepeat(_StructureBase):
         return fs or set([None])
     def get_checks(self, reverse):
         if self.min_count == 0:
-            return set()
+            return _Sequence()
         return self.subpattern.get_checks(reverse)
     def __eq__(self, other):
         return type(self) is type(other) and (self.subpattern, self.min_count, self.max_count) == (other.subpattern, other.min_count, other.max_count)
@@ -1833,7 +1817,7 @@ class _Group(_StructureBase):
     def contains_group(self):
         return True
     def compile(self, reverse=False):
-        return [(_OP.GROUP, self.group)] + self.subpattern.compile(reverse) + [(-_OP.GROUP, )]
+        return [(_OP.GROUP, self.group)] + self.subpattern.compile(reverse) + [(_OP.END, )]
     def remove_captures(self):
         return self.subpattern.remove_captures()
     def dump(self, indent=0, reverse=False):
@@ -1871,7 +1855,7 @@ class _LookAround(_StructureBase):
     def contains_group(self):
         return self.subpattern.contains_group()
     def compile(self, reverse=False):
-        return [(_OP.LOOKAROUND, int(self.positive), int(not self.behind))] + self.subpattern.compile(self.behind) + [(-_OP.LOOKAROUND, )]
+        return [(_OP.LOOKAROUND, int(self.positive), int(not self.behind))] + self.subpattern.compile(self.behind) + [(_OP.END, )]
     def dump(self, indent=0, reverse=False):
         print "%sLOOKAROUND %s %s" % (_INDENT * indent, self._dir_text[self.behind], self._pos_text[self.positive])
         self.subpattern.dump(indent + 1, self.behind)
@@ -2055,10 +2039,7 @@ class _Sequence(_StructureBase):
     def has_simple_start(self):
         return self.sequence and self.sequence[0].has_simple_start()
     def get_checks(self, reverse):
-        checks = set()
-        for subpattern in self.sequence:
-            checks |= subpattern.get_checks(reverse)
-        return checks
+        return _Sequence([subpattern.get_checks(reverse) for subpattern in self.sequence])
     @staticmethod
     def _flush_characters(char_type, characters, sequence):
         if not characters:
@@ -2174,7 +2155,7 @@ class _Set(_RegexBase):
         code.extend(self._make_bitset(ranges, False, False))
         for o in others:
             code.extend(o.compile())
-        code.append((-self._opcode[reverse], ))
+        code.append((_OP.END, ))
         return code
     def dump(self, indent=0, reverse=False):
         print "%s%s %s" % (_INDENT * indent, self._op_name[reverse], self._pos_text[self.positive])
@@ -2289,7 +2270,7 @@ class _String(_RegexBase):
     def has_simple_start(self):
         return True
     def get_checks(self, reverse):
-        return set([(False, tuple(self.characters))])
+        return self
     def get_first_char(self):
         raise error("internal error")
     def drop_first_char(self):
@@ -2300,8 +2281,6 @@ class _StringIgn(_String):
     _op_name = {False: "STRING_IGN", True: "STRING_IGN_REV"}
     def firstset(self):
         return set([_CharacterIgn(self.characters[0])])
-    def get_checks(self, reverse):
-        return set([(True, tuple(self.characters))])
 
 _all_char_types = (_Character, _CharacterIgn)
 _string_classes = {_Character: _String, _CharacterIgn: _StringIgn}
