@@ -297,7 +297,7 @@ def _compile(pattern, flags=0):
     parsed = parsed.optimise(info)
     parsed = parsed.pack_characters()
     reverse = bool(info.global_flags & REVERSE)
-    checks = parsed.get_checks(reverse)
+    checks = parsed.get_checks(reverse).optimise(info)
     parsed = _rc.prefix_checks(checks, parsed)
 
     # Should we print the parsed pattern?
